@@ -27,7 +27,8 @@
 #endif
 
 #include <kapplication.h>
-#include <kmainwindow.h>
+//#include <kmainwindow.h>
+#include <kmdimainfrm.h>
 #include <qtimer.h>
 #include <interfaces/guiinterface.h>
 
@@ -46,13 +47,15 @@ class KTorrentDCOP;
 class QLabel;
 class QListViewItem;
 class KTorrentPreferences;
-
+class KPushButton;
 
 
 namespace kt
 {
 	class TorrentInterface;
 	class ExpandableWidget;
+	class Group;
+	class GroupView;
 }
 
 /**
@@ -63,7 +66,7 @@ namespace kt
  * @author Joris Guisson <joris.guisson@gmail.com>
  * @version 0.1
  */
-class KTorrent : public KMainWindow, public kt::GUIInterface
+class KTorrent : public KMdiMainFrm, public kt::GUIInterface
 {
 	Q_OBJECT
 public:
@@ -86,19 +89,19 @@ public:
 	 */
 	void applySettings(bool change_port = true);
 
-	virtual void addTabPage(QWidget* page,const QIconSet & icon,const QString & caption);
+	virtual void addTabPage(QWidget* page,const QIconSet & icon,
+							const QString & caption,kt::CloseTabListener* ctl = 0);
 	virtual void removeTabPage(QWidget* page);
 	virtual void addPrefPage(kt::PrefPageInterface* page);
 	virtual void removePrefPage(kt::PrefPageInterface* page);
 	virtual void mergePluginGui(kt::Plugin* p);
 	virtual void removePluginGui(kt::Plugin* p);
-	virtual void addWidgetBelowView(QWidget* w);
+	virtual void addWidgetBelowView(QWidget* w,const QString & icon,const QString & caption);
 	virtual void removeWidgetBelowView(QWidget* w);
+	virtual void addToolWidget(QWidget* w,const QString & icon,const QString & caption,ToolDock dock);
+	virtual void removeToolWidget(QWidget* w);
 	virtual const kt::TorrentInterface* getCurrentTorrent() const;
 	
-	KTorrentView* getCurrentView();
-	kt::PanelView getCurrentPanel();
-
 	QString	getStatusInfo();
 	QString	getStatusTransfer();
 	QString	getStatusSpeed();
@@ -140,6 +143,7 @@ private slots:
 	void stopDownload();
 	void stopAllDownloads();
 	void queueAction();
+	void showIPFilter();
 	void removeDownload();
 	void queueManagerShow();
 	void optionsShowStatusbar();
@@ -149,13 +153,14 @@ private slots:
 	void newToolbarConfig();
 	void changeStatusbar(const QString& text);
 	void changeCaption(const QString& text);
-	void currentDownloadChanged(kt::TorrentInterface* tc);
-	void currentSeedChanged(kt::TorrentInterface* tc);
+	void currentTorrentChanged(kt::TorrentInterface* tc);
 	void updatedStats();
 	void urlDropped(QDropEvent*,QListViewItem*);
-	void currentTabChanged(QWidget* tab);
 	void onUpdateActions(bool can_start,bool can_stop,bool can_remove,bool can_scan);
 	void checkDataIntegrity();
+	void groupChanged(kt::Group* g);
+	void tabClosePressed();
+	void currentTabChanged(QWidget* w);
 	
 private:
 	void setupAccel();
@@ -164,27 +169,31 @@ private:
 	bool queryExit();
 	virtual void addWidgetInView(QWidget* w,kt::Position pos);
 	virtual void removeWidgetFromView(QWidget* w);
-	virtual void addWidgetInSeedView(QWidget* w,kt::Position pos);
-	virtual void removeWidgetFromSeedView(QWidget* w);
 	
 	
 private:
+	kt::GroupView* m_group_view;
 	KTorrentView *m_view;
-	KTorrentView *m_seedView;
 	KToggleAction *m_statusbarAction;
 	
 	KTorrentCore* m_core;
 	TrayIcon* m_systray_icon;
 	SetMaxRate* m_set_max_upload_rate;
 	SetMaxRate* m_set_max_download_rate;
-	KTabWidget* m_tabs;
+	
 	KTorrentDCOP* m_dcop;
 	QTimer m_gui_update_timer;
 	KTorrentPreferences* m_pref;
 	
-	kt::ExpandableWidget* m_exp;
+	struct Tab
+	{
+		KMdiChildView* child;
+		kt::CloseTabListener* ctl;
+	};
+	
 	kt::ExpandableWidget* m_view_exp;
-	kt::ExpandableWidget* m_seedView_exp;
+	QMap<QWidget*,Tab> m_tab_map;
+	KPushButton* m_close_cur_tab;
 
 	QLabel* m_statusInfo;
 	QLabel* m_statusTransfer;
@@ -200,6 +209,7 @@ private:
 	KAction* m_queuemgr; 
 	KAction* m_queueaction;
 	KAction* m_datacheck;
+	KAction* m_ipfilter;
 };
 
 #endif // _KTORRENT_H_
