@@ -41,22 +41,24 @@
 
 using namespace bt;
 
-/*
+
 void StupidWarningMessagesFromQt( QtMsgType type, const char *msg )
 {
-	switch ( type ) {
+	switch ( type ) 
+	{
 		case QtDebugMsg:
-			Out(SYS_GEN|LOG_DEBUG) << QString("Qt: Debug: %1").arg(msg) << endl;
+		//	printf("Qt: Debug: %s\n",msg);
 			break;
 		case QtWarningMsg:
-			Out(SYS_GEN|LOG_NOTICE) << QString("Qt: Warning: %1").arg(msg) << endl;
+			printf("Qt: Warning: %s\n",msg);
 			break;
 		case QtFatalMsg:
-			Out(SYS_GEN|LOG_IMPORTANT) << QString("Qt: Fatal: %1").arg(msg) << endl;
+			printf("Qt: Fatal : %s\n",msg);
 			abort();                    // deliberately core dump
+			break;
 	}
 }
-*/
+
 
 static const char description[] =
     I18N_NOOP("A BitTorrent program for KDE");
@@ -74,12 +76,14 @@ int main(int argc, char **argv)
 {
 	// ignore SIGPIPE's
 	signal(SIGPIPE,SIG_IGN);
-//	qInstallMsgHandler( StupidWarningMessagesFromQt );
+	qInstallMsgHandler( StupidWarningMessagesFromQt );
 	KAboutData about("ktorrent", I18N_NOOP("KTorrent"), kt::VERSION_STRING, description,
 					 KAboutData::License_GPL, "(C) 2005 -2006 Joris Guisson and Ivan Vasic", 0,
 					 "http://www.ktorrent.org/");
 	about.addAuthor("Joris Guisson", 0, "joris.guisson@gmail.com" );
 	about.addAuthor("Ivan Vasic",0,"ivasic@gmail.com");
+	about.addAuthor("Alan Jones",0,"skyphyr@gmail.com");
+	about.addAuthor("Diego R. Brogna",0,"dierbro@gmail.com");
 
 	about.addCredit("Mladen Babic",
 					I18N_NOOP("Application icon and a couple of others"),"bmladen@EUnet.yu");
@@ -98,6 +102,9 @@ int main(int argc, char **argv)
 	about.addCredit("Maxmind", I18N_NOOP("Country locator for InfoWidget plugin. Flags are taken from http://flags.blogpotato.de/ so thanks to them too."),0, "http://www.maxmind.com/");
 	about.addCredit("Adam Forsyth",I18N_NOOP("File prioritization"),"agforsyth@gmail.com");
 	about.addCredit("Thomas Bernard",I18N_NOOP("Miniupnp was used as an example for our own UPnP implementation"),0,"http://miniupnp.free.fr/");
+	about.addCredit("Diego Rosario Brogna",I18N_NOOP("Global max share ratio patch"),0,"dierbro@gmail.com");
+	about.addCredit("Lesly Weyts",I18N_NOOP("Zeroconf enhancements"),0,0);
+	about.addCredit("Kevin Andre",I18N_NOOP("Zeroconf enhancements"),0,"http://users.telenet.be/hyperquantum");
 
 	KCmdLineArgs::init(argc, argv, &about);
 	KCmdLineArgs::addCmdLineOptions(options);
