@@ -26,12 +26,11 @@
 namespace kt
 {
 	const bt::Uint32 MAJOR = 2;
-	const bt::Uint32 MINOR = 0;
-	const bt::Uint32 RELEASE = 3;
-	const char VERSION_STRING[] = "2.0.3";
-	const char PEER_ID[] = "-KT2030-";
+	const bt::Uint32 MINOR = 1;
+	const char VERSION_STRING[] = "2.1";
+	const char PEER_ID[] = "-KT2100-";
 }
 
-
+#define KT_VERSION_MACRO "2.1"
 
 #endif
