@@ -25,6 +25,7 @@
 
 namespace bt
 {
+	class SHA1Hash;
 
 	/**
 	 * @author Joris Guisson
@@ -32,7 +33,7 @@ namespace bt
 	 * 
 	 * Keeps track of a piece of the file. The Chunk has 3 possible states :
 	 * - MMAPPED : It is memory mapped
-	 * - BUFFERED : It is in a buffer in dynamicly allocated memory 
+	 * - BUFFERED : It is in a buffer in dynamically allocated memory 
 	 *  (because the chunk is located in 2 or more separate files, so we cannot just set a pointer
 	 *   to a region of mmapped memory)
 	 * - ON_DISK : On disk
@@ -99,12 +100,23 @@ namespace bt
 
 		/// Is chunk excluded
 		bool isExcluded() const;
+		
+		/// Is this a seed only chunk
+		bool isExcludedForDownloading() const;
 
 		/// In/Exclude chunk
 		void setExclude(bool yes);
 		
+		/**
+		 * Check wehter the chunk matches it's hash.
+		 * @param h The hash
+		 * @return true if the data matches the hash
+		 */
+		bool checkHash(const SHA1Hash & h) const;
+		
 		virtual void unmapped(bool remap_intended);
 		virtual void remapped(void* ptr);
+		
 	private:
 		Status status;
 		Uint32 index;
@@ -137,7 +149,15 @@ namespace bt
 	inline Priority Chunk::getPriority() const {return priority;}
 	inline void Chunk::setPriority(Priority newpriority) {priority = newpriority;}
 	inline bool Chunk::isExcluded() const 
-		{if(priority == EXCLUDED) return true; else return false;}
+	{
+		return priority == EXCLUDED; 
+	}
+	
+	inline bool Chunk::isExcludedForDownloading() const
+	{
+		return priority == ONLY_SEED_PRIORITY;
+	}
+	
 	inline void Chunk::setExclude(bool yes)
 		{if(yes) priority = EXCLUDED; else priority = NORMAL_PRIORITY;}
 }
