@@ -30,6 +30,7 @@
 #include "peer.h"
 #include "piece.h"
 #include "peerdownloader.h"
+#include <interfaces/functions.h>
 #include <interfaces/monitorinterface.h>
 #include "packetwriter.h"
 #include "chunkselector.h"
@@ -47,7 +48,8 @@ namespace bt
 		chunk_selector = new ChunkSelector(cman,*this,pman);
 		Uint64 total = tor.getFileLength();
 		downloaded = (total - cman.bytesLeft());
-		curr_chunks_dowloaded = 0;
+		curr_chunks_downloaded = 0;
+		unnecessary_data = 0;
 	
 		current_chunks.setAutoDelete(true);
 		connect(&pman,SIGNAL(newPeer(Peer* )),this,SLOT(onNewPeer(Peer* )));
@@ -62,52 +64,77 @@ namespace bt
 	
 	void Downloader::pieceRecieved(const Piece & p)
 	{
-		if (cman.chunksLeft() == 0)
+		if (cman.completed())
 			return;
 		
+		ChunkDownload* cd = 0;
+		
 		for (CurChunkItr j = current_chunks.begin();j != current_chunks.end();++j)
 		{
 			if (p.getIndex() != j->first)
 				continue;
 			
-			ChunkDownload* cd = j->second;
-			// if the chunk is not in memory, reload it
-			if (cd->getChunk()->getStatus() == Chunk::ON_DISK)
-			{
-				cman.prepareChunk(cd->getChunk(),true);
-			}
+			cd = j->second;
+			break;
+		}
+		
+		if (!cd)
+		{
+			unnecessary_data += p.getLength();
+			Out(SYS_DIO|LOG_DEBUG) << 
+					"Unnecessary piece, total unnecessary data : " << kt::BytesToString(unnecessary_data) << endl;
+			return;
+		}
+		
+		// if the chunk is not in memory, reload it
+		if (cd->getChunk()->getStatus() == Chunk::ON_DISK)
+		{
+			cman.prepareChunk(cd->getChunk(),true);
+		}
 			
-			downloaded += p.getLength();
-			if (cd->piece(p))
+		bool ok = false;
+		
+		if (cd->piece(p,ok))
+		{
+			if (tmon)
+				tmon->downloadRemoved(cd);
+				
+			if (ok)
+				downloaded += p.getLength();
+			
+			if (!finished(cd))
 			{
-				if (tmon)
-					tmon->downloadRemoved(cd);
-
-				if (!finished(cd))
-				{
-					// if the chunk fails don't count the bytes downloaded
-					if (cd->getChunk()->getSize() > downloaded)
-						downloaded = 0;
-					else
-						downloaded -= cd->getChunk()->getSize();
-				}
-				current_chunks.erase(p.getIndex());
-				return;
+				// if the chunk fails don't count the bytes downloaded
+				if (cd->getChunk()->getSize() > downloaded)
+					downloaded = 0;
+				else
+					downloaded -= cd->getChunk()->getSize();
 			}
-			else
+			current_chunks.erase(p.getIndex());
+		}
+		else
+		{
+			if (ok)
+				downloaded += p.getLength();
+			
+			// save to disk again, if it is idle
+			if (cd->isIdle() && cd->getChunk()->getStatus() == Chunk::MMAPPED)
 			{
-				// save to disk again, if it is idle
-				if (cd->isIdle() && cd->getChunk()->getStatus() == Chunk::MMAPPED)
-				{
-					cman.saveChunk(cd->getChunk()->getIndex(),false);
-				}
+				cman.saveChunk(cd->getChunk()->getIndex(),false);
 			}
 		}
+			
+		if (!ok)
+		{
+			unnecessary_data += p.getLength();
+			Out(SYS_DIO|LOG_DEBUG) << 
+					"Unnecessary piece, total unnecessary data : " << kt::BytesToString(unnecessary_data) << endl; 
+		}
 	}
 	
 	void Downloader::update()
 	{
-		if (cman.chunksLeft() == 0)
+		if (cman.completed())
 			return;
 		
 		/*
@@ -167,12 +194,7 @@ namespace bt
 			
 			if (ok)
 			{
-				if (pd->isChoked())
-				{
-					if (pd->getNumAllowedFastChunks() > 0)
-						downloadFromAF(pd);
-				}
-				else
+				if (!pd->isChoked())
 					downloadFrom(pd);
 				
 				pd->setNearlyDone(false);
@@ -183,26 +205,18 @@ namespace bt
 	Uint32 Downloader::maxMemoryUsage()
 	{
 		Uint32 max = 1024 * 1024;
-		if (cman.getNumChunks() - cman.chunksLeft() <= 4)
-		{
-			// in the beginning stick to four chunks for warmup mode
-			max = tor.getChunkSize() * 4;
-		}
-		else
-		{
-			switch (mem_usage)
-			{	
-				case 1: // Medium
-					max *= 20; // 20 MB
-					break;
-				case 2: // High
-					max *= 40; // 40 MB
-					break;
-				case 0: // LOW
-				default:
-					max *= 10; // 10 MB
-					break;
-			}
+		switch (mem_usage)
+		{	
+			case 1: // Medium
+				max *= 60; // 60 MB
+				break;
+			case 2: // High
+				max *= 80; // 90 MB
+				break;
+			case 0: // LOW
+			default:
+				max *= 40; // 30 MB
+				break;
 		}
 		return max;
 	}
@@ -219,39 +233,45 @@ namespace bt
 		return num_non_idle;
 	}
 	
-	bool Downloader::findDownloadForPD(PeerDownloader* pd)
+	ChunkDownload* Downloader::selectCD(PeerDownloader* pd,Uint32 num)
 	{
 		ChunkDownload* sel = 0;
 		Uint32 sel_left = 0xFFFFFFFF;
 		
-		// first see if there are ChunkDownload's which need a PeerDownloader
 		for (CurChunkItr j = current_chunks.begin();j != current_chunks.end();++j)
 		{
 			ChunkDownload* cd = j->second;
-			if (!pd->hasChunk(cd->getChunk()->getIndex()))
+			if (pd->isChoked() || !pd->hasChunk(cd->getChunk()->getIndex()))
 				continue;
 			
-			// if the peer is choked we only try allowed fast chunks
-			if (pd->isChoked())
+			if (cd->getNumDownloaders() == num) 
 			{
-				if (pd->inAllowedFastChunks(cd->getChunk()->getIndex()))
-				{
-					sel = cd;
-					break; // allowed fast so break
-				}
-			}
-			else
-			{
-				// if cd hasn't got a downloader we can try it
 				// lets favor the ones which are nearly finished
-				if (cd->getNumDownloaders() == 0 && 
-					(!sel || cd->getTotalPieces() - cd->getPiecesDownloaded() < sel_left))
+				if (!sel || cd->getTotalPieces() - cd->getPiecesDownloaded() < sel_left)
 				{
 					sel = cd;
 					sel_left = sel->getTotalPieces() - sel->getPiecesDownloaded();
 				}
 			}
 		}
+		return sel;
+	}
+	
+	bool Downloader::findDownloadForPD(PeerDownloader* pd,bool warmup)
+	{
+		ChunkDownload* sel = 0;
+		
+		// first see if there are ChunkDownload's which need a PeerDownloader
+		sel = selectCD(pd,0);
+		
+		if (!sel && warmup)
+		{
+			// if we couldn't find one, try to select another 
+			// which only has one downloader
+			// so that during warmup, there are at the most 2 downloaders 
+			// assigned to one peer	
+			sel = selectCD(pd,1);
+		}
 		
 		if (sel)
 		{
@@ -262,8 +282,29 @@ namespace bt
 			sel->assignPeer(pd);
 			return true;
 		}
+		
 		return false;
 	}
+	
+	ChunkDownload* Downloader::selectWorst(PeerDownloader* pd)
+	{
+		ChunkDownload* cdmin = NULL;
+		for (CurChunkItr j = current_chunks.begin();j != current_chunks.end();++j) 
+		{ 
+			ChunkDownload* cd = j->second; 
+			if (!pd->hasChunk(cd->getChunk()->getIndex()) || cd->containsPeer(pd))
+				continue;
+			 
+			if (!cdmin) 
+				cdmin = cd;
+			else if (cd->getDownloadSpeed() < cdmin->getDownloadSpeed())
+				cdmin = cd;
+			else if (cd->getNumDownloaders() < cdmin->getNumDownloaders()) 
+				cdmin = cd; 
+		}
+		 
+		return cdmin;
+	}
 
 	void Downloader::downloadFrom(PeerDownloader* pd)
 	{
@@ -273,10 +314,11 @@ namespace bt
 		Uint32 num_non_idle = numNonIdle();
 		
 		// first see if we can use an existing dowload
-		if (findDownloadForPD(pd))
+		if (findDownloadForPD(pd,cman.getNumChunks() - cman.chunksLeft() <= 4))
 			return;
 		
 		bool limit_exceeded = num_non_idle * tor.getChunkSize() >= max;
+		
 		Uint32 chunk = 0;
 		if (!limit_exceeded && chunk_selector->select(pd,chunk))
 		{
@@ -293,23 +335,8 @@ namespace bt
 		else if (pd->getNumGrabbed() == 0)
 		{ 
 			// If the peer hasn't got a chunk we want, 
-			// try to assign it to a chunk we are currently downloading 
-			// but we only do this if it hasn't been assigned to anything
-			ChunkDownload *cdmin=NULL; 
-			for (CurChunkItr j = current_chunks.begin();j != current_chunks.end();++j) 
-			{ 
-				ChunkDownload* cd = j->second; 
-				if (pd->hasChunk(cd->getChunk()->getIndex())) 
-				{ 
-					if (cd->containsPeer(pd)) 
-						continue; 
-					if (cdmin==NULL) 
-						cdmin=cd; 
-					else if (cd->getNumDownloaders()<cdmin->getNumDownloaders()) 
-						cdmin=cd; 
- 
-				} 
-			} 
+			ChunkDownload *cdmin = selectWorst(pd); 
+			
 			if (cdmin) 
 			{
 				// if it is on disk, reload it
@@ -323,42 +350,6 @@ namespace bt
 		} 
 	}
 	
-	void Downloader::downloadFromAF(PeerDownloader* pd)
-	{
-		// calculate the max memory usage
-		Uint32 max = maxMemoryUsage();
-		// calculate number of non idle chunks
-		Uint32 num_non_idle = numNonIdle();
-		
-		// first see if we can use an existing dowload
-		if (findDownloadForPD(pd))
-			return;
-		
-		bool limit_exceeded = num_non_idle * tor.getChunkSize() >= max;
-		if (limit_exceeded)
-			return;
-		
-		// try to find a chunk in the allowed fast set
-		AllowedFastSet::const_iterator itr = pd->beginAF();
-		while (itr != pd->endAF())
-		{
-			if (pd->hasChunk(*itr) && !current_chunks.contains(*itr))
-			{
-				Uint32 chunk = *itr;
-				Chunk* c = cman.getChunk(chunk);
-				if (cman.prepareChunk(c))
-				{
-					ChunkDownload* cd = new ChunkDownload(c);
-					current_chunks.insert(chunk,cd);
-					cd->assignPeer(pd);
-					if (tmon)
-						tmon->downloadStarted(cd);
-					return;
-				}	
-			}
-			itr++;
-		}
-	}
 
 	bool Downloader::areWeDownloading(Uint32 chunk) const
 	{
@@ -389,14 +380,19 @@ namespace bt
 	{
 		Chunk* c = cd->getChunk();
 		// verify the data
-		SHA1Hash h = SHA1Hash::generate(c->getData(),c->getSize());
+		SHA1Hash h;
+		if (cd->usingContinuousHashing())
+			h = cd->getHash();
+		else
+			h = SHA1Hash::generate(c->getData(),c->getSize());
+
 		if (tor.verifyHash(h,c->getIndex()))
 		{
 			// hash ok so save it
 			try
 			{
 				cman.saveChunk(c->getIndex());
-				Out() << "Chunk " << c->getIndex() << " downloaded " << endl;
+				Out(SYS_GEN|LOG_NOTICE) << "Chunk " << c->getIndex() << " downloaded " << endl;
 				// tell everybody we have the Chunk
 				for (Uint32 i = 0;i < pman.getNumConnectedPeers();i++)
 				{
@@ -405,16 +401,16 @@ namespace bt
 			}
 			catch (Error & e)
 			{
-				Out() << "Error " << e.toString() << endl;
+				Out(SYS_DIO|LOG_IMPORTANT) << "Error " << e.toString() << endl;
 				emit ioError(e.toString());
 				return false;
 			}
 		}
 		else
 		{
-			Out() << "Hash verification error on chunk "  << c->getIndex() << endl;
-			Out() << "Is        : " << h << endl;
-			Out() << "Should be : " << tor.getHash(c->getIndex()) << endl;
+			Out(SYS_GEN|LOG_IMPORTANT) << "Hash verification error on chunk "  << c->getIndex() << endl;
+			Out(SYS_GEN|LOG_IMPORTANT) << "Is        : " << h << endl;
+			Out(SYS_GEN|LOG_IMPORTANT) << "Should be : " << tor.getHash(c->getIndex()) << endl;
 			
 			cman.resetChunk(c->getIndex());
 			chunk_selector->reinsert(c->getIndex());
@@ -425,7 +421,7 @@ namespace bt
 				if (!p)
 					return false;
 				QString IP(p->getIPAddresss());
-				Out() << "Peer " << IP << " sent bad data" << endl;
+				Out(SYS_GEN|LOG_NOTICE) << "Peer " << IP << " sent bad data" << endl;
 				IPBlocklist & ipfilter = IPBlocklist::instance();
 				ipfilter.insert( IP );
 				p->kill(); 
@@ -502,7 +498,7 @@ namespace bt
 	void Downloader::loadDownloads(const QString & file)
 	{
 		// don't load stuff if download is finished
-		if (cman.chunksLeft() == 0)
+		if (cman.completed())
 			return;
 		
 		// Load all partial downloads
@@ -528,6 +524,11 @@ namespace bt
 			// first read header
 			fptr.read(&hdr,sizeof(ChunkDownloadHeader));
 			Out() << "Loading chunk " << hdr.index << endl;
+			if (hdr.index >= tor.getNumChunks())
+			{
+				Out() << "Warning : current_chunks file corrupted, invalid index " << hdr.index << endl;
+				return;
+			}
 			
 			if (!cman.getChunk(hdr.index) || current_chunks.contains(hdr.index))
 			{
@@ -535,7 +536,7 @@ namespace bt
 				return;
 			}
 			Chunk* c = cman.getChunk(hdr.index);
-			if (!c->isExcluded() && cman.prepareChunk(c))
+			if (!c->isExcluded() && !c->isExcludedForDownloading() && cman.prepareChunk(c))
 			{
 				ChunkDownload* cd = new ChunkDownload(c);
 				current_chunks.insert(hdr.index,cd);
@@ -548,7 +549,7 @@ namespace bt
 		}
 		
 		// reset curr_chunks_downloaded to 0
-		curr_chunks_dowloaded = 0;
+		curr_chunks_downloaded = 0;
 	}
 	
 	Uint32 Downloader::getDownloadedBytesOfCurrentChunksFile(const QString & file)
@@ -597,13 +598,13 @@ namespace bt
 			if (hdr.buffered)
 				fptr.seek(File::CURRENT,c->getSize());
 		}
-		curr_chunks_dowloaded = num_bytes;
+		curr_chunks_downloaded = num_bytes;
 		return num_bytes;
 	}
 
 	bool Downloader::isFinished() const
 	{
-		return cman.chunksLeft() == 0;
+		return cman.completed();
 	}
 
 	void Downloader::onExcluded(Uint32 from,Uint32 to)
@@ -628,6 +629,11 @@ namespace bt
 		chunk_selector->reincluded(from,to);
 	}
 	
+	void Downloader::corrupted(Uint32 chunk)
+	{
+		chunk_selector->reinsert(chunk);
+	}
+	
 	Uint32 Downloader::mem_usage = 0;
 	
 	void Downloader::setMemoryUsage(Uint32 m)
