@@ -20,7 +20,7 @@
 #include <math.h>
 #include <util/log.h>
 #include <util/functions.h>
-
+#include <net/address.h>
 #include <mse/streamsocket.h>
 
 #include "peer.h"
@@ -31,6 +31,10 @@
 #include "packetwriter.h"
 #include "peerdownloader.h"
 #include "peeruploader.h"
+#include "bdecoder.h"
+#include "bnode.h"
+#include "utpex.h"
+#include "server.h"
 
 using namespace net;
 
@@ -42,12 +46,14 @@ namespace bt
 	static Uint32 peer_id_counter = 1;
 	
 	
-	Peer::Peer(mse::StreamSocket* sock,const PeerID & peer_id,Uint32 num_chunks,Uint32 chunk_size,Uint32 support)
+	Peer::Peer(mse::StreamSocket* sock,const PeerID & peer_id,
+			   Uint32 num_chunks,Uint32 chunk_size,Uint32 support,bool local)
 	: sock(sock),pieces(num_chunks),peer_id(peer_id)
 	{
 		id = peer_id_counter;
 		peer_id_counter++;
 		
+		ut_pex = 0;
 		preader = new PacketReader(this);
 		choked = am_choked = true;
 		interested = am_interested = false;
@@ -63,7 +69,7 @@ namespace bt
 		connect_time = QTime::currentTime();
 		//sock->attachPeer(this);
 		stats.client = peer_id.identifyClient();
-		stats.ip_addresss = getIPAddresss();
+		stats.ip_address = getIPAddresss();
 		stats.choked = true;
 		stats.download_rate = 0;
 		stats.upload_rate = 0;
@@ -71,13 +77,15 @@ namespace bt
 		stats.snubbed = false;
 		stats.dht_support = support & DHT_SUPPORT;
 		stats.fast_extensions = support & FAST_EXT_SUPPORT;
+		stats.extension_protocol = support & EXT_PROT_SUPPORT;
 		stats.bytes_downloaded = stats.bytes_uploaded = 0;
 		stats.aca_score = 0.0;
 		stats.evil = false;
 		stats.has_upload_slot = false;
 		stats.num_up_requests = stats.num_down_requests = 0;
 		stats.encrypted = sock->encrypted();
-		if (stats.ip_addresss == "0.0.0.0")
+		stats.local = local;
+		if (stats.ip_address == "0.0.0.0")
 		{
 			Out(SYS_CON|LOG_DEBUG) << "No more 0.0.0.0" << endl;
 			kill();
@@ -86,11 +94,14 @@ namespace bt
 		{
 			sock->startMonitoring(preader,pwriter);
 		}
+		pex_allowed = stats.extension_protocol;
+		utorrent_pex_id = 0;
 	}
 
 
 	Peer::~Peer()
 	{
+		delete ut_pex;
 		delete uploader;
 		delete downloader;
 		delete sock;
@@ -198,9 +209,6 @@ namespace bt
 				
 				pieces = BitSet(tmp_buf+1,pieces.getNumBits());
 				bitSetRecieved(pieces);
-				// less then 5 pieces and support for fast_extensions so enable allowed fast
-				if (stats.fast_extensions && pieces.numOnBits() < 5)
-					uploader->enableAllowedFast();
 				break;
 			case REQUEST:
 				if (len != 13)
@@ -310,26 +318,70 @@ namespace bt
 				}
 				pieces.setAll(false);
 				bitSetRecieved(pieces);
-				// no pieces so give enable AF if peer supports it
-				if (stats.fast_extensions)
-					uploader->enableAllowedFast();
 				break;
 			case SUGGEST_PIECE:
 				// ignore suggestions for the moment
 				break;
 			case ALLOWED_FAST:
-				if (len != 5)
-				{
-					Out() << "len err ALLOWED_FAST" << endl;
-					kill();
-					return;
-				}
-				downloader->addAllowedFastChunk(ReadUint32(tmp_buf,1));
+				// we no longer support this, so do nothing
+				break;
+			case EXTENDED:
+				handleExtendedPacket(packet,len);
 				break;
 		}
 	}
 	
-	Uint32 Peer::sendData(const Uint8* data,Uint32 len,bool proto)
+	void Peer::handleExtendedPacket(const Uint8* packet,Uint32 size)
+	{
+		if (size <= 2 || packet[1] > 1)
+			return;
+		
+		if (packet[1] == 1)
+		{
+			if (ut_pex)
+				ut_pex->handlePexPacket(packet,size);
+			return;
+		}
+		
+		QByteArray tmp;
+		tmp.setRawData((const char*)packet,size);
+		BNode* node = 0;
+		try
+		{
+			BDecoder dec(tmp,false,2);
+			node = dec.decode();
+			if (node && node->getType() == BNode::DICT)
+			{
+				BDictNode* dict = (BDictNode*)node;
+				
+				// handshake packet, so just check if the peer supports ut_pex
+				dict = dict->getDict("m");
+				BValueNode* val = 0;
+				if (dict && (val = dict->getValue("ut_pex")))
+				{
+					utorrent_pex_id = val->data().toInt();
+					if (ut_pex && utorrent_pex_id == 0)
+					{
+						delete ut_pex;
+						ut_pex = 0;
+					}
+					else if (!ut_pex)
+						ut_pex = new UTPex(this,utorrent_pex_id);
+					else
+						ut_pex->changeID(utorrent_pex_id);
+				}
+			}
+		}
+		catch (...)
+		{
+			// just ignore invalid packets
+			Out(SYS_CON|LOG_DEBUG) << "Invalid extended packet" << endl;
+		}
+		delete node;
+		tmp.resetRawData((const char*)packet,size);
+	}
+	
+	Uint32 Peer::sendData(const Uint8* data,Uint32 len)
 	{
 		if (killed) return 0;
 		
@@ -357,7 +409,7 @@ namespace bt
 		return sock->bytesAvailable();
 	}
 
-	void Peer::dataWritten(int bytes)
+	void Peer::dataWritten(int )
 	{
 	//	Out() << "dataWritten " << bytes << endl;
 		
@@ -384,7 +436,7 @@ namespace bt
 		return true;
 	}
 	
-	void Peer::update()
+	void Peer::update(PeerManager* pman)
 	{
 		if (killed)
 			return;
@@ -399,19 +451,21 @@ namespace bt
 		preader->update();
 		
 		Uint32 data_bytes = pwriter->getUploadedDataBytes();
-		Uint32 non_data_bytes = pwriter->getUploadedNonDataBytes();
 		
 		if (data_bytes > 0)
 		{
 			stats.bytes_uploaded += data_bytes;
 			uploader->addUploadedBytes(data_bytes);
 		}
+		
+		if (ut_pex && ut_pex->needsUpdate())
+			ut_pex->update(pman);
 	}
 	
 	bool Peer::isSnubbed() const
 	{
 		// 4 minutes
-		return snub_timer.getElapsedSinceUpdate() >= 2*60*1000;
+		return snub_timer.getElapsedSinceUpdate() >= 2*60*1000 && stats.num_down_requests > 0;
 	}
 
 	bool Peer::isSeeder() const
@@ -422,10 +476,26 @@ namespace bt
 	QString Peer::getIPAddresss() const
 	{
 		if (sock)
-			return sock->getIPAddress();
+			return sock->getRemoteIPAddress();
 		else
 			return QString::null;
 	}
+	
+	Uint16 Peer::getPort() const
+	{
+		if (!sock)
+			return 0;
+		else
+			return sock->getRemotePort();
+	}
+	
+	net::Address Peer::getAddress() const
+	{
+		if (!sock)
+			return net::Address();
+		else
+			return sock->getRemoteAddress();
+	}
 
 	Uint32 Peer::getTimeSinceLastPiece() const
 	{
@@ -460,15 +530,42 @@ namespace bt
 			return;
 		
 		pwriter->sendChoke();
-		if (stats.fast_extensions)
+		uploader->clearAllRequests();
+	}
+	
+	void Peer::emitPortPacket()
+	{
+		gotPortPacket(sock->getRemoteIPAddress(),sock->getRemotePort());
+	}
+	
+	void Peer::emitPex(const QByteArray & data)
+	{
+		pex(data);
+	}
+	
+	void Peer::setPexEnabled(bool on)
+	{
+		if (!stats.extension_protocol)
+			return;
+		
+		// send extension protocol handshake
+		bt::Uint16 port = Globals::instance().getServer().getPortInUse();
+		
+		if (ut_pex && !on)
 		{
-			// send rejects for all queued up packets
-			uploader->rejectAll();
+			delete ut_pex;
+			ut_pex = 0;
+			
 		}
-		else
+		else if (!ut_pex && on && utorrent_pex_id > 0)
 		{
-			uploader->clearAllRequests();
+			// if the other side has enabled it to, create a new UTPex object
+			ut_pex = new UTPex(this,utorrent_pex_id);
 		}
+		
+		pwriter->sendExtProtHandshake(port,on);
+		
+		pex_allowed = on;
 	}
 }
 
