@@ -73,10 +73,11 @@ namespace bt
 		connect(peer,SIGNAL(piece(const Piece& )),this,SLOT(piece(const Piece& )));
 		connect(peer,SIGNAL(destroyed()),this,SLOT(peerDestroyed()));
 		nearly_done = false;
+		max_wait_queue_size = 25;
 	}
 
 
-	PeerDownloader::~PeerDownloader()
+	PeerDownloader::~PeerDownloader() 
 	{
 	}
 #if 0
@@ -90,7 +91,7 @@ namespace bt
 
 	bool PeerDownloader::canAddRequest() const
 	{
-		return wait_queue.count() < 50;
+		return wait_queue.count() < max_wait_queue_size;
 	}
 
 	Uint32 PeerDownloader::getNumRequests() const 
@@ -141,8 +142,8 @@ namespace bt
 		if (!peer)
 			return;
 
-		Out(SYS_CON|LOG_DEBUG) << "Rejected : " << req.getIndex() << " " 
-				<< req.getOffset() << " " << req.getLength() << endl;
+//		Out(SYS_CON|LOG_DEBUG) << "Rejected : " << req.getIndex() << " " 
+//				<< req.getOffset() << " " << req.getLength() << endl;
 		if (reqs.contains(req))
 		{
 			reqs.remove(req);
@@ -210,13 +211,14 @@ namespace bt
 	
 	void PeerDownloader::checkTimeouts()
 	{
+		TimeStamp now = bt::GetCurrentTime(); 
 		// we use a 60 second interval
 		const Uint32 MAX_INTERVAL = 60 * 1000;
 		QValueList<TimeStampedRequest>::iterator i = reqs.begin();
 		while (i != reqs.end())
 		{
 			TimeStampedRequest & tr = *i;
-			if (bt::GetCurrentTime() - tr.time_stamp > MAX_INTERVAL)
+			if (now - tr.time_stamp > MAX_INTERVAL)
 			{
 			//	Out() << "Request " << tr.req.getIndex() << " " << tr.req.getOffset() << " timed out !" << endl;
 				// cancel it
@@ -237,51 +239,22 @@ namespace bt
 		}
 	}
 	
-	void PeerDownloader::addAllowedFastChunk(Uint32 chunk)
-	{
-		allowed_fast.insert(chunk);
-	}
 	
-	bool PeerDownloader::inAllowedFastChunks(Uint32 chunk)
-	{
-		return allowed_fast.count(chunk) > 0;
-	}
-	
-	/*
-	Uint32 PeerDownloader::getMaximumOutstandingReqs() const
-	{
-		// get the download rate in KB/sec
-		double pieces_per_sec = (double)peer->getDownloadRate() / MAX_PIECE_LEN;
-		
-		if (pieces_per_sec < 1.0)
-			return 5;
-		else
-			return 5 + (Uint32)ceil(2*pieces_per_sec);
-	}
-	*/
-
 	Uint32 PeerDownloader::getMaxChunkDownloads() const
 	{
 		// get the download rate in KB/sec
-		Uint32 rate_kbs = peer->getDownloadRate() / 1024;
-		Uint32 num_extra = 0;
-		if (rate_kbs >= 50)
-			num_extra = 1;
-		else if (rate_kbs >= 100)
-			num_extra = 2;
-		else if (rate_kbs >= 150)
-			num_extra = 3;
-		
-		// also take into account the size of each chunk
-		// if a chunk has less then 16 pieces we multiply by 16 / num_pieces_per_chunk
-		Uint32 mul_factor = 1;
+		Uint32 rate_kbs = peer->getDownloadRate();
+		rate_kbs = rate_kbs / 1024;
+		Uint32 num_extra = rate_kbs / 50;
 		
 		if (chunk_size >= 16)
-			mul_factor = 1;
+		{
+			return 1 + 16 * num_extra / chunk_size;
+		}
 		else
-			mul_factor = 16 / chunk_size;
-		 
-		return mul_factor * (1 + num_extra);
+		{
+			return 1 + (16 / chunk_size) * num_extra;
+		}
 	}
 	
 	void PeerDownloader::choked()
@@ -290,27 +263,19 @@ namespace bt
 		while (i != reqs.end())
 		{
 			TimeStampedRequest & tr = *i;
-			if (allowed_fast.count(tr.req.getIndex()) == 0)
-			{
-				rejected(tr.req);
-				i = reqs.erase(i);
-			}
-			else
-				i++;
+			rejected(tr.req);
+			i++;
 		}
+		reqs.clear();
 		
 		QValueList<Request>::iterator j = wait_queue.begin();
 		while (j != wait_queue.end())
 		{
 			Request & req = *j;
-			if (allowed_fast.count(req.getIndex()) == 0)
-			{
-				rejected(req);
-				j = wait_queue.erase(j);
-			}
-			else
-				j++;
+			rejected(req);
+			j++;
 		}
+		wait_queue.clear();
 	}
 	
 	void PeerDownloader::update()
@@ -318,7 +283,7 @@ namespace bt
 		// modify the interval if necessary
 		double pieces_per_sec = (double)peer->getDownloadRate() / MAX_PIECE_LEN;
 		
-		Uint32 max_reqs = 1 + (Uint32)ceil(pieces_per_sec);
+		Uint32 max_reqs = 1 + (Uint32)ceil(8*pieces_per_sec);
 		
 		while (wait_queue.count() > 0 && reqs.count() < max_reqs)
 		{
@@ -329,6 +294,10 @@ namespace bt
 			reqs.append(r);
 			peer->getPacketWriter().sendRequest(req);
 		}
+		
+		max_wait_queue_size = 2*max_reqs;
+		if (max_wait_queue_size < 10)
+			max_wait_queue_size = 10;
 	}
 }
 
