@@ -36,28 +36,21 @@
 namespace bt
 {
 
-	Torrent::Torrent() : piece_length(0),file_length(0),anon_list(0),priv_torrent(false)
+	Torrent::Torrent() : piece_length(0),file_length(0),priv_torrent(false)
 	{
 		encoding = "utf8";
+		trackers = 0;
 	}
 
 
 	Torrent::~Torrent()
 	{
-		delete anon_list;
+		delete trackers;
 	}
-
-	void Torrent::load(const QString & file,bool verbose)
+	
+	
+	void Torrent::load(const QByteArray & data,bool verbose)
 	{
-		QFile fptr(file);
-		if (!fptr.open(IO_ReadOnly))
-			throw Error(i18n(" Unable to open torrent file %1 : %2")
-					.arg(file).arg(fptr.errorString()));
-		
-		QByteArray data(fptr.size());
-	//	Out() << "File size = " << fptr.size() << endl;
-		fptr.readBlock(data.data(),fptr.size());
-		
 		BNode* node = 0;
 		 
 		try
@@ -76,13 +69,17 @@ namespace bt
 				Out() << "Encoding : " << encoding << endl;
 			}
 
-			if (dict->getValue("announce"))
-				loadTrackerURL(dict->getValue("announce"));
-			else if (dict->getList("nodes")) // DHT torrrents have a node key
-				loadNodes(dict->getList("nodes"));
-			else
+			BValueNode* announce = dict->getValue("announce");
+			BListNode* nodes = dict->getList("nodes");
+			if (!announce && !nodes)
 				throw Error(i18n("Torrent has no announce or nodes field"));
 				
+			if (announce)
+				loadTrackerURL(announce);
+			
+			if (nodes) // DHT torrrents have a node key
+				loadNodes(nodes);
+			
 			loadInfo(dict->getDict("info"));
 			loadAnnounceList(dict->getData("announce-list"));
 			BNode* n = dict->getData("info");
@@ -97,6 +94,20 @@ namespace bt
 			throw;
 		}
 	}
+
+	void Torrent::load(const QString & file,bool verbose)
+	{
+		QFile fptr(file);
+		if (!fptr.open(IO_ReadOnly))
+			throw Error(i18n(" Unable to open torrent file %1 : %2")
+					.arg(file).arg(fptr.errorString()));
+		
+		QByteArray data(fptr.size());
+	//	Out() << "File size = " << fptr.size() << endl;
+		fptr.readBlock(data.data(),fptr.size());
+		
+		load(data,verbose);
+	}
 	
 	void Torrent::loadInfo(BDictNode* dict)
 	{
@@ -115,6 +126,17 @@ namespace bt
 		n = dict->getValue("private");
 		if (n && n->data().toInt() == 1)
 			priv_torrent = true;
+		
+		// do a safety check to see if the number of hashes matches the file_length
+		Uint32 num_chunks = (file_length / this->piece_length);
+		if (file_length % piece_length > 0)
+			num_chunks++;
+		
+		if (num_chunks != hash_pieces.count())
+		{
+			Out(SYS_GEN|LOG_DEBUG) << "File sizes and number of hashes do not match for " << name_suggestion << endl;
+			throw Error(i18n("Corrupted torrent!"));
+		}
 	}
 	
 	void Torrent::loadFiles(BListNode* node)
@@ -161,7 +183,6 @@ namespace bt
 
 				// update file_length
 				file_length += s;
-			
 				files.append(file);
 			}
 			else
@@ -177,7 +198,11 @@ namespace bt
 		if (!node || node->data().getType() != Value::STRING)
 			throw Error(i18n("Corrupted torrent!"));
 		
-		tracker_url = node->data().toString(encoding).stripWhiteSpace();
+		// tracker_urls.append(KURL(node->data().toString(encoding).stripWhiteSpace()));
+		if (!trackers)
+			trackers = new TrackerTier();
+		
+		trackers->urls.append(KURL(node->data().toString(encoding).stripWhiteSpace()));
 	}
 	
 	void Torrent::loadPieceLength(BValueNode* node)
@@ -211,7 +236,7 @@ namespace bt
 		if (!node || node->data().getType() != Value::STRING)
 			throw Error(i18n("Corrupted torrent!"));
 		
-
+		
 		QByteArray hash_string = node->data().toByteArray();
 		for (unsigned int i = 0;i < hash_string.size();i+=20)
 		{
@@ -235,14 +260,34 @@ namespace bt
 		if (!node)
 			return;
 		
-		if (anon_list)
-		{
-			delete anon_list;
-			anon_list = 0;
-		}
+		BListNode* ml = dynamic_cast<BListNode*>(node);
+		if (!ml)
+			return;
 		
-		anon_list = new AnnounceList();
-		anon_list->load(node);
+		if (!trackers)
+			trackers = new TrackerTier();
+		
+		TrackerTier* tier = trackers;
+		//ml->printDebugInfo();
+		for (Uint32 i = 0;i < ml->getNumChildren();i++)
+		{
+			BListNode* url = dynamic_cast<BListNode*>(ml->getChild(i));
+			if (!url)
+				throw Error(i18n("Parse Error"));
+			
+			for (Uint32 j = 0;j < url->getNumChildren();j++)
+			{
+				BValueNode* vn = dynamic_cast<BValueNode*>(url->getChild(j));
+				if (!vn)
+					throw Error(i18n("Parse Error"));
+
+				KURL url(vn->data().toString().stripWhiteSpace());
+				tier->urls.append(url);
+				//Out() << "Added tracker " << url << endl;
+			}
+			tier->next = new TrackerTier();
+			tier = tier->next;
+		}
 	}
 	
 	void Torrent::loadNodes(BListNode* node)
@@ -276,10 +321,9 @@ namespace bt
 	void Torrent::debugPrintInfo()
 	{
 		Out() << "Name : " << name_suggestion << endl;
-		if (!anon_list)
-			Out() << "Tracker URL : " << tracker_url << endl;
-		else
-			anon_list->debugPrintURLList();
+		
+//		for (KURL::List::iterator i = tracker_urls.begin();i != tracker_urls.end();i++)
+//			Out() << "Tracker URL : " << *i << endl;
 		
 		Out() << "Piece Length : " << piece_length << endl;
 		if (this->isMultiFile())
@@ -337,21 +381,17 @@ namespace bt
 		
 		return files.at(idx);
 	}
-	
-	KURL Torrent::getTrackerURL(bool last_was_succesfull) const
-	{
-		if (anon_list)
-			return anon_list->getTrackerURL(last_was_succesfull);
-		else
-			return tracker_url;
-	}
 
 	unsigned int Torrent::getNumTrackerURLs() const
 	{
-		if (anon_list)
-			return anon_list->getNumTrackerURLs();
-		else
-			return 1;
+		Uint32 count = 0;
+		TrackerTier* tt = trackers;
+		while (tt)
+		{
+			count += tt->urls.count();
+			tt = tt->next;
+		}
+		return count;
 	}
 
 	void Torrent::calcChunkPos(Uint32 chunk,QValueList<Uint32> & file_list) const
@@ -373,16 +413,6 @@ namespace bt
 		return IsMultimediaFile(this->getNameSuggestion());
 	}
 	
-	AnnounceList* Torrent::createAnnounceList()
-	{
-		if(!anon_list)
-			anon_list = new AnnounceList();
-		
-		anon_list->addTracker(tracker_url, false);
-
-		return anon_list;
-	}
-	
 	void Torrent::updateFilePercentage(const BitSet & bs)
 	{
 		for (Uint32 i = 0;i < files.count();i++)
